Package: rstanarm
Type: Package
Title: Bayesian Applied Regression Modeling via Stan
Version: 2.32.2
Date: 2025-09-29
Encoding: UTF-8
Authors@R: c(person("Jonah", "Gabry", email = "jsg2201@columbia.edu", role = "aut"),
             person("Imad", "Ali", role = "ctb"),
             person("Sam", "Brilleman", role = "ctb"),
             person(given = "Jacqueline Buros", family = "Novik",
                    role = "ctb", comment = "R/stan_jm.R"),
             person("AstraZeneca", role = "ctb", comment = "R/stan_jm.R"),
             person("Trustees of", "Columbia University", role = "cph"),
             person("Simon", "Wood", role = "cph", comment = "R/stan_gamm4.R"),
             person("R Core", "Deveopment Team", role = "cph",
                     comment = "R/stan_aov.R"),
             person("Douglas", "Bates", role = "cph", comment = "R/pp_data.R"),
             person("Martin", "Maechler", role = "cph", comment = "R/pp_data.R"),
             person("Ben", "Bolker", role = "cph", comment = "R/pp_data.R"),
             person("Steve", "Walker", role = "cph", comment = "R/pp_data.R"),
             person("Brian", "Ripley", role = "cph",
                    comment = "R/stan_aov.R, R/stan_polr.R"),
             person("William", "Venables", role = "cph", comment = "R/stan_polr.R"),
             person("Paul-Christian", "Burkner", email = "paul.buerkner@gmail.com",
                    role = "cph", comment = "R/misc.R"),
             person("Ben", "Goodrich", email = "benjamin.goodrich@columbia.edu",
                    role = c("cre", "aut")))
Description: Estimates previously compiled regression models using the 'rstan'
    package, which provides the R interface to the Stan C++ library for Bayesian
    estimation. Users specify models via the customary R syntax with a formula and
    data.frame plus some additional arguments for priors.
License: GPL (>= 3)
Depends: R (>= 3.4.0), Rcpp (>= 0.12.0), methods
Imports: bayesplot (>= 1.7.0), ggplot2 (>= 2.2.1), lme4 (>= 1.1-8), loo
        (>= 2.1.0), Matrix (>= 1.2-13), nlme (>= 3.1-124), posterior,
        rstan (>= 2.32.0), rstantools (>= 2.1.0), shinystan (>= 2.3.0),
        stats, survival (>= 2.40.1), RcppParallel (>= 5.0.1), utils,
        reformulas
Suggests: biglm, betareg, data.table (>= 1.10.0), digest, gridExtra,
        HSAUR3, knitr (>= 1.15.1), MASS, mgcv (>= 1.8-13), rmarkdown,
        roxygen2, StanHeaders (>= 2.21.0), testthat (>= 1.0.2), gamm4,
        shiny, V8
LinkingTo: StanHeaders (>= 2.32.0), rstan (>= 2.32.0), BH (>=
        1.72.0-2), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1)
SystemRequirements: GNU make, pandoc (>= 1.12.3), pandoc-citeproc
VignetteBuilder: knitr
LazyData: true
UseLTO: true
NeedsCompilation: yes
URL: https://mc-stan.org/rstanarm/, https://discourse.mc-stan.org
BugReports: https://github.com/stan-dev/rstanarm/issues
RoxygenNote: 7.3.3
Packaged: 2025-09-29 18:05:27 UTC; ben
Author: Jonah Gabry [aut],
  Imad Ali [ctb],
  Sam Brilleman [ctb],
  Jacqueline Buros Novik [ctb] (R/stan_jm.R),
  AstraZeneca [ctb] (R/stan_jm.R),
  Trustees of Columbia University [cph],
  Simon Wood [cph] (R/stan_gamm4.R),
  R Core Deveopment Team [cph] (R/stan_aov.R),
  Douglas Bates [cph] (R/pp_data.R),
  Martin Maechler [cph] (R/pp_data.R),
  Ben Bolker [cph] (R/pp_data.R),
  Steve Walker [cph] (R/pp_data.R),
  Brian Ripley [cph] (R/stan_aov.R, R/stan_polr.R),
  William Venables [cph] (R/stan_polr.R),
  Paul-Christian Burkner [cph] (R/misc.R),
  Ben Goodrich [cre, aut]
Maintainer: Ben Goodrich <benjamin.goodrich@columbia.edu>
Repository: CRAN
Date/Publication: 2025-09-30 08:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-30 07:42:03 UTC; unix
Archs: rstanarm.so.dSYM
