# Generated by roxygen2: do not edit by hand

S3method(VarCorr,stanreg)
S3method(as.array,stanreg)
S3method(as.data.frame,stanreg)
S3method(as.data.frame,summary.stanreg)
S3method(as.matrix,stanreg)
S3method(as_draws,stanreg)
S3method(as_draws_array,stanreg)
S3method(as_draws_df,stanreg)
S3method(as_draws_list,stanreg)
S3method(as_draws_matrix,stanreg)
S3method(as_draws_rvars,stanreg)
S3method(bayes_R2,stanreg)
S3method(coef,stanmvreg)
S3method(coef,stanreg)
S3method(confint,stanreg)
S3method(family,stanmvreg)
S3method(family,stanreg)
S3method(fitted,stanmvreg)
S3method(fitted,stanreg)
S3method(fixef,stanmvreg)
S3method(fixef,stanreg)
S3method(formula,stanmvreg)
S3method(formula,stanreg)
S3method(get_x,betareg)
S3method(get_x,default)
S3method(get_x,gamm4)
S3method(get_x,lmerMod)
S3method(get_x,stanmvreg)
S3method(get_y,default)
S3method(get_y,stanmvreg)
S3method(get_z,lmerMod)
S3method(get_z,stanmvreg)
S3method(kfold,stanreg)
S3method(launch_shinystan,stanreg)
S3method(log_lik,stanjm)
S3method(log_lik,stanmvreg)
S3method(log_lik,stanreg)
S3method(loo,stanreg)
S3method(loo_R2,stanreg)
S3method(loo_compare,stanreg)
S3method(loo_compare,stanreg_list)
S3method(loo_linpred,stanreg)
S3method(loo_model_weights,stanreg_list)
S3method(loo_predict,stanreg)
S3method(loo_predictive_interval,stanreg)
S3method(model.frame,stanmvreg)
S3method(model.frame,stanreg)
S3method(model.matrix,stanreg)
S3method(names,stanreg_list)
S3method(ngrps,stanmvreg)
S3method(ngrps,stanreg)
S3method(nobs,stanmvreg)
S3method(nobs,stanreg)
S3method(nsamples,stanreg)
S3method(pairs,stanreg)
S3method(plot,predict.stanjm)
S3method(plot,stanreg)
S3method(plot,survfit.stanjm)
S3method(posterior_epred,stanreg)
S3method(posterior_interval,stanreg)
S3method(posterior_linpred,stanreg)
S3method(posterior_predict,stanmvreg)
S3method(posterior_predict,stanreg)
S3method(posterior_vs_prior,stanreg)
S3method(pp_check,stanreg)
S3method(predict,stanreg)
S3method(predictive_error,matrix)
S3method(predictive_error,ppd)
S3method(predictive_error,stanreg)
S3method(predictive_interval,matrix)
S3method(predictive_interval,ppd)
S3method(predictive_interval,stanreg)
S3method(print,compare_rstanarm_loos)
S3method(print,prior_summary.stanreg)
S3method(print,stanmvreg)
S3method(print,stanreg)
S3method(print,stanreg_list)
S3method(print,summary.stanmvreg)
S3method(print,summary.stanreg)
S3method(print,survfit.stanjm)
S3method(prior_summary,stanreg)
S3method(ranef,stanmvreg)
S3method(ranef,stanreg)
S3method(residuals,stanmvreg)
S3method(residuals,stanreg)
S3method(se,stanmvreg)
S3method(se,stanreg)
S3method(sigma,stanmvreg)
S3method(sigma,stanreg)
S3method(summary,stanmvreg)
S3method(summary,stanreg)
S3method(terms,stanmvreg)
S3method(terms,stanreg)
S3method(update,stanjm)
S3method(update,stanmvreg)
S3method(update,stanreg)
S3method(vcov,stanreg)
S3method(waic,stanreg)
export(R2)
export(Surv)
export(VarCorr)
export(as_draws)
export(as_draws_array)
export(as_draws_df)
export(as_draws_list)
export(as_draws_matrix)
export(as_draws_rvars)
export(bayes_R2)
export(cauchy)
export(compare_models)
export(decov)
export(default_prior_coef)
export(default_prior_intercept)
export(dirichlet)
export(exponential)
export(fixef)
export(get_x)
export(get_y)
export(get_z)
export(hs)
export(hs_plus)
export(invlogit)
export(kfold)
export(laplace)
export(lasso)
export(launch_shinystan)
export(lkj)
export(log_lik)
export(logit)
export(loo)
export(loo_R2)
export(loo_compare)
export(loo_linpred)
export(loo_model_weights)
export(loo_predict)
export(loo_predictive_interval)
export(neg_binomial_2)
export(ngrps)
export(normal)
export(nsamples)
export(pairs_condition)
export(pairs_style_np)
export(plot_nonlinear)
export(plot_stack_jm)
export(posterior_epred)
export(posterior_interval)
export(posterior_linpred)
export(posterior_predict)
export(posterior_survfit)
export(posterior_traj)
export(posterior_vs_prior)
export(pp_check)
export(pp_validate)
export(predictive_error)
export(predictive_interval)
export(prior_options)
export(prior_summary)
export(product_normal)
export(ps_check)
export(ranef)
export(se)
export(sigma)
export(stan_aov)
export(stan_betareg)
export(stan_betareg.fit)
export(stan_biglm)
export(stan_biglm.fit)
export(stan_clogit)
export(stan_gamm4)
export(stan_glm)
export(stan_glm.fit)
export(stan_glm.nb)
export(stan_glmer)
export(stan_glmer.nb)
export(stan_jm)
export(stan_lm)
export(stan_lm.fit)
export(stan_lm.wfit)
export(stan_lmer)
export(stan_mvmer)
export(stan_nlmer)
export(stan_polr)
export(stan_polr.fit)
export(stanjm_list)
export(stanmvreg_list)
export(stanreg_list)
export(student_t)
export(waic)
if(getRversion()>='3.3.0') importFrom(stats, sigma) else
  importFrom(lme4,sigma)

import(Rcpp)
import(bayesplot)
import(methods)
import(rstantools)
import(shinystan)
import(stats)
importFrom(Matrix,Matrix)
importFrom(Matrix,t)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(bayesplot,pairs_condition)
importFrom(bayesplot,pairs_style_np)
importFrom(ggplot2,"%+replace%")
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_contour)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_pointrange)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_step)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,labs)
importFrom(ggplot2,rel)
importFrom(ggplot2,scale_color_gradient2)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(lme4,fixef)
importFrom(lme4,glFormula)
importFrom(lme4,glmer)
importFrom(lme4,glmerControl)
importFrom(lme4,lmer)
importFrom(lme4,lmerControl)
importFrom(lme4,mkVarCorr)
importFrom(lme4,ngrps)
importFrom(lme4,nlformula)
importFrom(lme4,nlmer)
importFrom(lme4,nlmerControl)
importFrom(lme4,ranef)
importFrom(loo,is.kfold)
importFrom(loo,is.loo)
importFrom(loo,is.waic)
importFrom(loo,kfold)
importFrom(loo,loo)
importFrom(loo,loo.function)
importFrom(loo,loo.matrix)
importFrom(loo,loo_compare)
importFrom(loo,loo_model_weights)
importFrom(loo,psis)
importFrom(loo,waic)
importFrom(loo,waic.function)
importFrom(loo,waic.matrix)
importFrom(nlme,VarCorr)
importFrom(posterior,as_draws)
importFrom(posterior,as_draws_array)
importFrom(posterior,as_draws_df)
importFrom(posterior,as_draws_list)
importFrom(posterior,as_draws_matrix)
importFrom(posterior,as_draws_rvars)
importFrom(reformulas,findbars)
importFrom(rstan,constrain_pars)
importFrom(rstan,extract)
importFrom(rstan,extract_sparse_parts)
importFrom(rstan,get_posterior_mean)
importFrom(rstan,optimizing)
importFrom(rstan,sampling)
importFrom(rstan,stanc)
importFrom(rstan,vb)
importFrom(rstantools,loo_R2)
importFrom(rstantools,nsamples)
importFrom(stats,cov2cor)
importFrom(stats,getInitial)
importFrom(survival,Surv)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,tail)
importMethodsFrom(rstan,summary)
useDynLib(rstanarm, .registration = TRUE)
